# Copyright 2014 SolidBuilds.com. All rights reserved
#
# Authors: Ling Thio <ling.thio@gmail.com>

from docassemble.webapp.app_and_db import app
from flask import render_template, current_app
from docassemble_flask_user import login_required, roles_required

# The Member page is accessible to authenticated users (users that have logged in)
@app.route('/member')
@login_required             # Limits access to authenticated users
def member_page():
    return render_template('pages/member_page.html')

# The Admin page is accessible to users with the 'admin' role
@app.route('/admin')
@roles_required('admin')    # Limits access to users with the 'admin' role
def admin_page():
    return render_template('pages/admin_page.html')




import docassemble.base.config
import docassemble.webapp.user_database
import sys

if __name__ == "__main__":
    docassemble.base.config.load()
    if len(sys.argv) > 1:
        db_config = sys.argv[1]
    else:
        db_config = 'db'
    print(docassemble.webapp.user_database.alchemy_url(db_config))



"""alter database for mysql compatibility

Revision ID: 9be372ec38bc
Revises: 4328f2c08f05
Create Date: 2020-02-16 15:43:35.276655

"""
from alembic import op
import sqlalchemy as sa
from docassemble.webapp.database import dbtableprefix, dbprefix, daconfig
import sys

# revision identifiers, used by Alembic.
revision = '9be372ec38bc'
down_revision = '4328f2c08f05'
branch_labels = None
depends_on = None

def upgrade():
    if dbprefix.startswith('postgresql') and not daconfig.get('force text to varchar upgrade', False):
        sys.stderr.write("Not changing text type to varchar type because underlying database is PostgreSQL\n")
    else:
        op.alter_column(
            table_name='userdict',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='userdictkeys',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='chatlog',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='uploads',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='uploads',
            column_name='yamlfile',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='objectstorage',
            column_name='key',
            type_=sa.String(1024)
        )
        op.alter_column(
            table_name='speaklist',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='shortener',
            column_name='filename',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='shortener',
            column_name='key',
            type_=sa.String(255)
        )
        op.alter_column(
            table_name='machinelearning',
            column_name='key',
            type_=sa.String(1024)
        )
        op.alter_column(
            table_name='machinelearning',
            column_name='group_id',
            type_=sa.String(1024)
        )
        op.alter_column(
            table_name='globalobjectstorage',
            column_name='key',
            type_=sa.String(1024)
        )
    op.create_index(dbtableprefix + 'ix_uploads_yamlfile', 'uploads', ['yamlfile'])

def downgrade():
    op.alter_column(
        table_name='userdict',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='userdictkeys',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='chatlog',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='uploads',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='uploads',
        column_name='yamlfile',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='objectstorage',
        column_name='key',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='speaklist',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='shortener',
        column_name='filename',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='shortener',
        column_name='key',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='machinelearning',
        column_name='key',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='machinelearning',
        column_name='group_id',
        type_=sa.Text()
    )
    op.alter_column(
        table_name='globalobjectstorage',
        column_name='key',
        type_=sa.Text()
    )
    op.drop_index(dbtableprefix + 'ix_uploads_yamlfile', table_name='uploads')



#import sys
try:
    from werkzeug.middleware.proxy_fix import ProxyFix
    proxyfix_version = 15
except ImportError:
    from werkzeug.contrib.fixers import ProxyFix
    proxyfix_version = 14
from flask import Flask
from flask_wtf.csrf import CSRFProtect
from flask_babel import Babel
from flask_cors import CORS

def create_app():
    app = Flask(__name__)
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    from docassemble.base.config import daconfig
    import docassemble.webapp.database
    import docassemble.webapp.db_object
    alchemy_connect_string = docassemble.webapp.database.alchemy_connection_string()
    app.config['SQLALCHEMY_DATABASE_URI'] = alchemy_connect_string
    if alchemy_connect_string.startswith('postgres'):
        app.config['SQLALCHEMY_ENGINE_OPTIONS'] = dict(connect_args=docassemble.webapp.database.connect_args())
    app.secret_key = daconfig.get('secretkey', '38ihfiFehfoU34mcq_4clirglw3g4o87')
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db = docassemble.webapp.db_object.init_flask()
    db.init_app(app)
    csrf = CSRFProtect()
    csrf.init_app(app)
    babel = Babel()
    babel.init_app(app)
    if daconfig.get('behind https load balancer', False):
        if proxyfix_version >= 15:
            app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)
        else:
            app.wsgi_app = ProxyFix(app.wsgi_app)
    if 'cross site domains' in daconfig:
        CORS(app, origins=daconfig['cross site domains'], supports_credentials=True)
    return app, csrf, babel

import docassemble.base.functions
if docassemble.base.functions.server_context.context == 'websockets':
    from docassemble.webapp.app_socket import app
else:
    app, csrf, flaskbabel = create_app()



from docassemble.webapp.db_object import db
from docassemble.base.config import daconfig, dbtableprefix
from docassemble.webapp.database import dbprefix
from sqlalchemy import true, false
import docassemble.webapp.users.models
from sqlalchemy.dialects.postgresql.json import JSONB

class Uploads(db.Model):
    __tablename__ = dbtableprefix + "uploads"
    indexno = db.Column(db.Integer(), primary_key=True)
    key = db.Column(db.String(250), index=True)
    filename = db.Column(db.String(255), index=True)
    yamlfile = db.Column(db.String(255), index=True)
    private = db.Column(db.Boolean(), nullable=False, server_default=true())
    persistent = db.Column(db.Boolean(), nullable=False, server_default=false())

class UploadsUserAuth(db.Model):
    __tablename__ = dbtableprefix + "uploadsuserauth"
    id = db.Column(db.Integer(), primary_key=True)
    uploads_indexno = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'uploads.indexno', ondelete='CASCADE'), nullable=False, index=True)
    user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'user.id', ondelete='CASCADE'), index=True)
    temp_user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'tempuser.id', ondelete='CASCADE'), index=True)

class UploadsRoleAuth(db.Model):
    __tablename__ = dbtableprefix + "uploadsroleauth"
    id = db.Column(db.Integer(), primary_key=True)
    uploads_indexno = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'uploads.indexno', ondelete='CASCADE'), nullable=False, index=True)
    role_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'role.id', ondelete='CASCADE'), nullable=False, index=True)

class ObjectStorage(db.Model):
    __tablename__ = dbtableprefix + "objectstorage"
    id = db.Column(db.Integer(), primary_key=True)
    key = db.Column(db.String(1024), index=True)
    value = db.Column(db.Text())

class SpeakList(db.Model):
    __tablename__ = dbtableprefix + "speaklist"
    id = db.Column(db.Integer(), primary_key=True)
    filename = db.Column(db.String(255), index=True)
    key = db.Column(db.String(250), index=True)
    phrase = db.Column(db.Text())
    question = db.Column(db.Integer())
    type = db.Column(db.String(20))
    language = db.Column(db.String(10))
    dialect = db.Column(db.String(10))
    upload = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'uploads.indexno', ondelete='CASCADE'))
    encrypted = db.Column(db.Boolean(), nullable=False, server_default=true())
    digest = db.Column(db.Text())

class Supervisors(db.Model):
    __tablename__ = dbtableprefix + "supervisors"
    id = db.Column(db.Integer(), primary_key=True)
    hostname = db.Column(db.Text())
    url = db.Column(db.Text())
    start_time = db.Column(db.DateTime(), server_default=db.func.now())
    role = db.Column(db.Text())

class MachineLearning(db.Model):
    __tablename__ = dbtableprefix + "machinelearning"
    id = db.Column(db.Integer(), primary_key=True)
    group_id = db.Column(db.String(1024))
    key = db.Column(db.String(1024), index=True)
    independent = db.Column(db.Text())
    dependent = db.Column(db.Text())
    info = db.Column(db.Text())
    create_time = db.Column(db.DateTime())
    modtime = db.Column(db.DateTime())
    active = db.Column(db.Boolean(), nullable=False, server_default=false())

class Shortener(db.Model):
    __tablename__ = dbtableprefix + "shortener"
    id = db.Column(db.Integer(), primary_key=True)
    short = db.Column(db.String(250), nullable=False, unique=True)
    filename = db.Column(db.String(255), index=True)
    uid = db.Column(db.String(250))
    user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'user.id', ondelete='CASCADE'))
    temp_user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'tempuser.id', ondelete='CASCADE'))
    key = db.Column(db.String(255), index=True)
    index = db.Column(db.Integer())
    modtime = db.Column(db.DateTime(), server_default=db.func.now())

class Email(db.Model):
    __tablename__ = dbtableprefix + "email"
    id = db.Column(db.Integer(), primary_key=True)
    short = db.Column(db.String(250), db.ForeignKey(dbtableprefix + 'shortener.short', ondelete='CASCADE'))
    all_addr = db.Column(db.Text())
    to_addr = db.Column(db.Text())
    cc_addr = db.Column(db.Text())
    from_addr = db.Column(db.Text())
    reply_to_addr = db.Column(db.Text())
    return_path_addr = db.Column(db.Text())
    subject = db.Column(db.Text())
    datetime_message = db.Column(db.DateTime())
    datetime_received = db.Column(db.DateTime())

class EmailAttachment(db.Model):
    __tablename__ = dbtableprefix + "emailattachment"
    id = db.Column(db.Integer(), primary_key=True)
    email_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'email.id', ondelete='CASCADE'))
    index = db.Column(db.Integer())
    content_type = db.Column(db.Text())
    extension = db.Column(db.Text())
    upload = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'uploads.indexno', ondelete='CASCADE'))
    
# class DbInfo(db.Model):
#     __tablename__ = dbtableprefix + "dbinfo"
#     key = db.Column(db.Text(), primary_key=True)
#     value = db.Column(db.Text())

class GlobalObjectStorage(db.Model):
    __tablename__ = dbtableprefix + "globalobjectstorage"
    id = db.Column(db.Integer(), primary_key=True)
    key = db.Column(db.String(1024), index=True)
    value = db.Column(db.Text())
    encrypted = db.Column(db.Boolean(), nullable=False, server_default=true())
    user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'user.id', ondelete='CASCADE'))
    temp_user_id = db.Column(db.Integer(), db.ForeignKey(dbtableprefix + 'tempuser.id', ondelete='CASCADE'))

class JsonStorage(db.Model):
    __tablename__ = dbtableprefix + "jsonstorage"
    id = db.Column(db.Integer(), primary_key=True)
    filename = db.Column(db.String(255), index=True)
    key = db.Column(db.String(250), index=True)
    if dbprefix.startswith('postgresql'):
        data = db.Column(JSONB)
    else:
        data = db.Column(db.Text())
    tags = db.Column(db.Text())
    modtime = db.Column(db.DateTime(), server_default=db.func.now())
    persistent = db.Column(db.Boolean(), nullable=False, server_default=false())



from minio import Minio
from minio.error import ResponseError, BucketAlreadyOwnedByYou, BucketAlreadyExists
import sys
import re

hostname = re.sub(r'^https?://', r'', sys.argv[1])

minioClient = Minio(hostname,
                    access_key=sys.argv[2],
                    secret_key=sys.argv[3],
                    secure=False)

try:
    minioClient.make_bucket(sys.argv[4])
except BucketAlreadyOwnedByYou as err:
    sys.stderr.write("Bucket already exists\n")
except BucketAlreadyExists as err:
    sys.stderr.write("Bucket already exists\n")
except ResponseError as err:
    raise



from docassemble.base.error import DAError
import docassemble.base.config
if not docassemble.base.config.loaded:
    docassemble.base.config.load()
from docassemble.base.config import daconfig
import os

if 'db' not in daconfig:
    daconfig['db'] = dict()
dbuser = daconfig['db'].get('user', None)
dbpassword = daconfig['db'].get('password', None)
dbhost = daconfig['db'].get('host', None)
if dbhost is None and dbuser is not None:
    dbhost = 'localhost'
dbport = daconfig['db'].get('port', None)
dbprefix = daconfig['db'].get('prefix', 'postgresql+psycopg2://')
dbname = daconfig['db'].get('name', 'docassemble')
dbtableprefix = daconfig['db'].get('table prefix', None)
if not dbtableprefix:
    dbtableprefix = ''

connect_string = ""
if dbname is not None:
    connect_string += "dbname=" + dbname
else:
    raise DAError("No database name provided")
if dbuser is not None:
    connect_string += " user=" + dbuser
if dbpassword is not None:
    connect_string += " password=" + dbpassword

pool_pre_ping = daconfig.get('sql ping', False)

alchemy_connect_string = ""
if dbprefix is not None:
    alchemy_connect_string += dbprefix
if dbuser is not None:
    alchemy_connect_string += dbuser
if dbpassword is not None:
    alchemy_connect_string += ":" + dbpassword
else:
    alchemy_connect_string += ":"
if dbhost is not None:
    alchemy_connect_string += '@' + dbhost
    if dbport is not None:
        alchemy_connect_string += ':' + str(dbport)
else:
    alchemy_connect_string += '@'
if not dbprefix.startswith('oracle'):
    if dbname is not None:
        alchemy_connect_string += "/" + dbname
    else:
        raise DAError("No database name provided")

alchemy_connect_args = dict()
if alchemy_connect_string.startswith('postgres'):
    ssl_mode = daconfig['db'].get('ssl mode', None)
    if ssl_mode in ('disable', 'allow', 'prefer', 'require', 'verify-ca', 'verify-full'):
        alchemy_connect_args['sslmode'] = ssl_mode
    for local_parameter, postgres_parameter in (('ssl cert', 'sslcert'), ('ssl key', 'sslkey'), ('ssl root cert', 'sslrootcert')):
        filename = daconfig['db'].get(local_parameter, None)
        if isinstance(filename, str):
            cert_file = os.path.join(daconfig.get('web server certificate directory', '/var/www/.certs'), filename)
            if os.path.isfile(cert_file):
                alchemy_connect_args[postgres_parameter] = cert_file

def connection_string():
    return connect_string

def alchemy_connection_string():
    return alchemy_connect_string

def connect_args():
    return alchemy_connect_args



import sys
import os
import docassemble.base.config

if __name__ == "__main__":
    docassemble.base.config.load(arguments=sys.argv)

def main():
    from docassemble.base.config import daconfig
    webapp_path = daconfig.get('webapp', '/usr/share/docassemble/webapp/docassemble.wsgi')
    wsgi_file = webapp_path
    if os.path.isfile(wsgi_file):
        with open(wsgi_file, 'a'):
            os.utime(wsgi_file, None)
            sys.stderr.write("Restarted WSGI.\n")
    sys.exit(0)

if __name__ == "__main__":
    main()



import sys

if __name__ == "__main__":
    import docassemble.base.config
    docassemble.base.config.load(arguments=sys.argv)

def check_for_config():
    from docassemble.base.config import daconfig
    import docassemble.webapp.cloud
    cloud = docassemble.webapp.cloud.get_cloud()
    if cloud is not None:
        key = cloud.get_key('config.yml')
        if key.does_exist:
            key.get_contents_to_filename(daconfig['config file'])

if __name__ == "__main__":
    check_for_config()

            

